------------------------------------------------------------------------------
--                                                                          --
--                         GNAT COMPILER COMPONENTS                         --
--                                                                          --
--                              C O M P E R R                               --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--                            $Revision: 1.9 $                              --
--                                                                          --
--           Copyright (c) 1992,1993,1994 NYU, All Rights Reserved          --
--                                                                          --
-- GNAT is free software;  you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion.  GNAT is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with GNAT;  see file COPYING.  If not, write --
-- to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. --
--                                                                          --
------------------------------------------------------------------------------

--  This package contains the routine called when a fatal internal compiler
--  error is detected. Calls to this routines cause termination of the
--  current compilation with appropriate error output.

with Types; use Types;

package Comperr is

   Fatal_Error_Node  : Node_Id;
   --  Set to the node giving the location at which the compiler abort
   --  is to be signalled. This is not necessarily perfectly accurate,
   --  it is just a convenient best guess.

   procedure Compiler_Abort (X : String; Code : Integer := 0);
   --  Signals an internal compiler error. Never returns control. Depending
   --  on processing may end up raising Unrecoverable_Error, or exiting
   --  directly. The message output is "internal error: X" where X is the
   --  string passed as an argument. The node in Fatal_Error_Node is used
   --  to provide the location where the error should be signalled. The
   --  message includes the node id, and the code parameter if it is non-zero)

end Comperr;


----------------------
-- REVISION HISTORY --
----------------------

--  ----------------------------
--  revision 1.7
--  date: Tue Jan 25 12:04:30 1994;  author: dewar
--  (Compiler_Abort): Slight change in calling sequence
--  (Fatal_Error_Loc): Contains location for Compiler_Abort error message
--  ----------------------------
--  revision 1.8
--  date: Sat Feb 12 13:26:35 1994;  author: dewar
--  Replace Fatal_Error_Loc by Fatal_Error_Node
--  ----------------------------
--  revision 1.9
--  date: Sun Feb 13 20:15:33 1994;  author: dewar
--  (Compiler_Abort): Add Code parameter, fix documentation
--  ----------------------------
--  New changes after this line.  Each line starts with: "--  "
